#include <pwd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <locale.h>
#include <langinfo.h>
#include <sys/stat.h>
#include "config.h"
#include "smbitem.h"

#define FIELD_MAX	3
#define	PATTERN_SIZE	20
#define LINE_SIZE	200

int split(char *line, char *arg[], int len[], int count){
    int i, l = 0, cnt = 0, mode = '\0', slash = 0;

    for(i = 0; i < count; i++) memset(arg[i], 0, len[i]);
    for( ; *line; line++){
	if (mode == '\0'){
	    if (*line == '#') return cnt;
	    if ((*line == ' ') || (*line == '\t')) continue;
	    cnt++; l = 0; slash = 0; mode = ' ';
	    if ((*line == '"') || (*line == '\'') || (*line == '`')){
		mode = *line;
		continue;
	    }
	}else
	if (mode == ' '){
	    if ((*line == ' ') || (*line == '\t')){
		mode = '\0';
		continue;
	    }
	}else{
	    if (!slash){
		if (*line == '\\'){
		    slash = 1;
		    continue;
		}else
		if (*line == mode){
		    if ((line[1] == ' ') || (line[1] == '\t') ||
					    (line[1] == '\0')){
			mode = '\0';
			continue;
		    }
		    return -cnt;
		}
	    }
	}
        slash = 0; l++; 
	if ((cnt <= count) && (len[cnt - 1] > l)) arg[cnt - 1][l - 1] = *line;
    }
    return cnt;
}


int read_config(char *filename, struct option *opt){
    FILE		*file;
    int			name_len, i, cnt, len[FIELD_MAX];
    char		*name, *link, pattern[PATTERN_SIZE], s[LINE_SIZE];
    char		data[FIELD_MAX][LINE_SIZE], *arg[FIELD_MAX];
    struct stat		buf;

    init_option(opt);
    for(i = 0; i < FIELD_MAX; i++){
	arg[i] = data[i];
	len[i] = sizeof(data[i]);
    }
    snprintf(pattern, sizeof(pattern), "%%%d[^\n]\n", sizeof(s) - 1);
    
    if (filename == NULL) return 0;
    if (lstat(filename, &buf) != 0){
	DPRINT("Error while reading config %s : %s.\n",
	    filename, strerror(errno));
	return 0;
    }
    if (!S_ISREG(buf.st_mode)){
	DPRINT("Config %s is not regular file.\n", filename);
	return 0;
    }
    if ((buf.st_mode & 06177) != 0){
	DPRINT("Insecure config file permission.\n"
	    "Run 'chmod 600 %s' to fix it.\n", filename);
	return 0;
    }

    if ((file = fopen(filename, "r")) == NULL){
	DPRINT("Open file %s error : %s", filename, strerror(errno));
	return 0;
    }
    fscanf(file, "%[\n]", s);
    while(!feof(file)){
	memset(s, 0, sizeof(s));
	fscanf(file, pattern, s);
	cnt = split(s, arg, len, FIELD_MAX);
	if (cnt == 0) continue;

	if ((cnt == 2) && set_option(opt, arg[0], arg[1])) continue;
	if ((cnt == 2) && (strcasecmp(arg[0], "host") == 0))
	    if (mkhost(arg[1], USERPATH)) continue;
	if ((cnt == 2) && (strcasecmp(arg[0], "group") == 0))
	    if (mkgroup(arg[1], USERPATH)) continue;
	if ((cnt >= 2) && (cnt <= 3) && (strcasecmp(arg[0], "link") == 0)){
	    if (strlen(arg[2]) != 0){
		if (mklink(arg[1], arg[2], USERPATH)) continue;
	    }else{
		name = (char*) GetSecondPath(arg[1], &name_len);
		if (name_len > 0){
		    if ((link = malloc(strlen(name) + 4)) != NULL){
			strcpy(link, "../");
			strcpy(link + 3, name);
			if (mklink(arg[1], link, USERPATH)){
			    free(link);
			    continue;
			}
			free(link);
		    }
		}
	    }
	}

	DPRINT("Invalid input line : %s\n", s);
    }
    fclose(file);
    check_option(opt);
    return 1;
}
