#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <pthread.h>
#include <iconv.h>
#include <libsmbclient.h>
#include "config.h"
#include "convert.h"
#include "smbdescr.h"
#include "smbitem.h"
#include "reconfigure.h"
#include "update.h"

pthread_t	t_autorefresh;
pthread_t	t_configupdate;

void UpdateGroup(char *group){
    struct SmbDescriptor	*descr;
    const char			*smbpath;
    struct smbc_dirent		*dirent;
    iconv_t			cd;
    char			*name, *path, *link;

    if ((smbpath = path2samba(group)) == NULL) return;
    if ((descr = GetSmbDescriptor(group, strlen(group))) == NULL)
	goto error1;
    if ((descr->fd = descr->ctx->opendir(descr->ctx, smbpath)) == NULL){
	descr->error = errno;
	goto error2;
    }
    
    cd = samba2local();
    while ((dirent = descr->ctx->readdir(descr->ctx, descr->fd)) != NULL){
	if (dirent->smbc_type != SMBC_SERVER) continue;
	if ((name = (char*)from_samba(dirent->name, cd)) == NULL) continue;

	if ((path = malloc(strlen(group) + strlen(name) + 2)) == NULL)
	    goto error3;
	strcpy(path, group);
	path[strlen(group)] = '/';
	strcpy(path + strlen(group) + 1, name);
	
	if ((link = malloc(strlen(name) + 4)) == NULL) goto error4;	
	strcpy(link, "../");
	strcpy(link + 3, name);
	
	mklink(path, link, AUTOPATH);
	
	free(link);
      error4:
	free(path);
      error3:
	free(name);
    }
    iconv_close(cd);

    descr->ctx->closedir(descr->ctx, descr->fd);
  error2:
    ReleaseSmbDescriptor(descr);
  error1:
    free((char*)smbpath);
}

void UpdateRoot(){
    struct SmbDescriptor	*descr;
    struct smbc_dirent		*dirent;
    struct smbitemarray		*array;
    char			*name;
    iconv_t			cd;
    int				i;

    if (!option(OPT_QUERY_BROWSERS)) goto error0;

    if ((descr = GetSmbDescriptor("", 0)) == NULL) goto error0;
    if ((descr->fd = descr->ctx->opendir(descr->ctx, "smb://")) == NULL){
	descr->error = errno;
	goto error1;
    }

    // this is safe, couse cd will be checked in convert function.
    cd = samba2local();
    while ((dirent = descr->ctx->readdir(descr->ctx, descr->fd)) != NULL){
	if (dirent->smbc_type != SMBC_WORKGROUP) continue;
	if ((name = (char*)from_samba(dirent->name, cd)) == NULL) continue;
	mkgroup(name, AUTOPATH);
	free(name);
    }
    iconv_close(cd);

    descr->ctx->closedir(descr->ctx, descr->fd);
  error1:
    ReleaseSmbDescriptor(descr);
  error0:

    if ((array = getdir("", AUTOPATH)) != NULL){
	for(i = 0; i < array->count; i++)
	    UpdateGroup((char*)array->elem[i]->name);
	ReleaseSmbItemArray(array);
    }
}

void* AutoRefreshThread(void *data){
    time_t	ctime1, ctime2;

    (void)data;

    ctime1 = 0;
    while(1){
	ctime2 = time(NULL);
	if (ctime2 - ctime1 > option(OPT_REFRESH_TIME)){
	    UpdateRoot();
	    ReleaseOldItem(ctime2 - option(OPT_DEAD_TIME), AUTOPATH);
	    ctime1 = ctime2;
	}
	sleep(5);
    }
    return NULL;
}

void* ConfigUpdateThread(void *data){
    time_t		ctime;
    struct stat		st1, st2;
    struct option	opt;

    (void)data;

    memset(&st1, 0, sizeof(struct stat));
    while(1){
	ctime = time(NULL);
	memcpy(&st2, &st1, sizeof(struct stat));
	stat(config, &st2);
	if ((st1.st_ino   != st2.st_ino) ||
	    (st1.st_size  != st2.st_size) ||
	    (st1.st_mtime != st2.st_mtime)){
	
	    if (read_config(config, &opt)) set_all_option(&opt);
	    ReleaseOldItem(ctime, USERPATH);
	    memcpy(&st1, &st2, sizeof(struct stat));
	}
	sleep(5);
    }
    return NULL;
}

int InitThreads(){
    if (pthread_create(&t_configupdate, NULL, 
	    ConfigUpdateThread, NULL) != 0) return 1;
    if (pthread_create(&t_autorefresh, NULL, 
	    AutoRefreshThread, NULL) != 0) return 1;
    return 0;
}

void DestroyThreads(){
    pthread_cancel(t_configupdate);
    pthread_join(t_configupdate, NULL);

    pthread_cancel(t_autorefresh);
    pthread_join(t_autorefresh, NULL);

    ReleaseOldItem(time(NULL) + 1, AUTOPATH | USERPATH); 
}
